#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class AccessType:
    INVALID: Enum
    BYTE: Enum
    SHORT: Enum
    WORD: Enum
    DOUBLEWORD: Enum
    QUADWORD: Enum
    FLOAT: Enum
    DOUBLEFLOAT: Enum
    WORD_LEFT: Enum
    WORD_RIGHT: Enum
    DOUBLEWORD_LEFT: Enum
    DOUBLEWORD_RIGHT: Enum
    MAX: Enum
