#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegGprO32:
    zero: Enum
    at: Enum
    v0: Enum
    v1: Enum
    a0: Enum
    a1: Enum
    a2: Enum
    a3: Enum
    t0: Enum
    t1: Enum
    t2: Enum
    t3: Enum
    t4: Enum
    t5: Enum
    t6: Enum
    t7: Enum
    s0: Enum
    s1: Enum
    s2: Enum
    s3: Enum
    s4: Enum
    s5: Enum
    s6: Enum
    s7: Enum
    t8: Enum
    t9: Enum
    k0: Enum
    k1: Enum
    gp: Enum
    sp: Enum
    fp: Enum
    ra: Enum
