#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


class JrRegData:
    def hasInfo(self) -> bool: ...
    def offset(self) -> int: ...
    def address(self) -> int: ...
    def checkedForBranching(self) -> bool: ...
    def lastBranchOffset(self) -> bool: ...
