#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class InstrIdType:
    ALL_INVALID: Enum
    CPU_INVALID: Enum
    CPU_NORMAL: Enum
    CPU_SPECIAL: Enum
    CPU_REGIMM: Enum
    CPU_COP0: Enum
    CPU_COP0_BC0: Enum
    CPU_COP0_TLB: Enum
    CPU_COP1: Enum
    CPU_COP1_BC1: Enum
    CPU_COP1_FPUS: Enum
    CPU_COP1_FPUD: Enum
    CPU_COP1_FPUW: Enum
    CPU_COP1_FPUL: Enum
    CPU_COP2: Enum
    RSP_INVALID: Enum
    RSP_NORMAL: Enum
    RSP_NORMAL_LWC2: Enum
    RSP_NORMAL_SWC2: Enum
    RSP_SPECIAL: Enum
    RSP_REGIMM: Enum
    RSP_COP0: Enum
    RSP_COP1: Enum
    RSP_COP2: Enum
    RSP_COP2_VU: Enum
    R3000GTE_INVALID: Enum
    R3000GTE_NORMAL: Enum
    R3000GTE_SPECIAL: Enum
    R3000GTE_REGIMM: Enum
    R3000GTE_COP0: Enum
    R3000GTE_COP1: Enum
    R3000GTE_COP2: Enum
    R3000GTE_COP2_GTE: Enum
    R4000ALLEGREX_INVALID: Enum
    R4000ALLEGREX_NORMAL: Enum
    R4000ALLEGREX_SPECIAL: Enum
    R4000ALLEGREX_SPECIAL_RS: Enum
    R4000ALLEGREX_SPECIAL_SA: Enum
    R4000ALLEGREX_REGIMM: Enum
    R4000ALLEGREX_SPECIAL2: Enum
    R4000ALLEGREX_SPECIAL3: Enum
    R4000ALLEGREX_SPECIAL3_BSHFL: Enum
    R4000ALLEGREX_COP0: Enum
    R4000ALLEGREX_COP0_BC0: Enum
    R4000ALLEGREX_COP0_TLB: Enum
    R4000ALLEGREX_COP1: Enum
    R4000ALLEGREX_COP1_BC1: Enum
    R4000ALLEGREX_COP1_FPUS: Enum
    R4000ALLEGREX_COP1_FPUW: Enum
    R4000ALLEGREX_COP2: Enum
    R4000ALLEGREX_COP2_BC2: Enum
    R4000ALLEGREX_COP2_MFHC2: Enum
    R4000ALLEGREX_COP2_MFHC2_P: Enum
    R4000ALLEGREX_COP2_MFHC2_P_S: Enum
    R4000ALLEGREX_COP2_MTHC2: Enum
    R4000ALLEGREX_VFPU0: Enum
    R4000ALLEGREX_VFPU1: Enum
    R4000ALLEGREX_VFPU3: Enum
    R4000ALLEGREX_VFPU4: Enum
    R4000ALLEGREX_VFPU4_FMT0: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT0: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT2: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT3: Enum
    R4000ALLEGREX_VFPU4_FMT0_RND: Enum
    R4000ALLEGREX_VFPU4_FMT0_CVTFLT: Enum
    R4000ALLEGREX_VFPU4_FMT0_CVTINT: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT8: Enum
    R4000ALLEGREX_VFPU4_FMT0_FMT9: Enum
    R4000ALLEGREX_VFPU4_FMT0_CONTROL: Enum
    R4000ALLEGREX_VFPU4_FMT0_COLOR: Enum
    R4000ALLEGREX_VFPU4_FMT0_CST: Enum
    R4000ALLEGREX_VFPU4_FMT2: Enum
    R4000ALLEGREX_VFPU4_FMT2_CNDMOVE: Enum
    R4000ALLEGREX_VFPU5: Enum
    R4000ALLEGREX_VFPU6: Enum
    R4000ALLEGREX_VFPU6_FMT7: Enum
    R4000ALLEGREX_VFPU6_FMT7_FMT0: Enum
    R4000ALLEGREX_VFPU7: Enum
    R4000ALLEGREX_QUADLR: Enum
    R5900_INVALID: Enum
    R5900_NORMAL: Enum
    R5900_SPECIAL: Enum
    R5900_REGIMM: Enum
    R5900_COP0: Enum
    R5900_COP0_TLB: Enum
    R5900_COP1: Enum
    R5900_COP1_FPUS: Enum
    R5900_COP2: Enum
    R5900_COP2_NOHIGHBIT: Enum
    R5900_COP2_BC2: Enum
    R5900_COP2_SPECIAL1: Enum
    R5900_COP2_SPECIAL2: Enum
    R5900_COP2_VIWR: Enum
    R5900_MMI: Enum
    R5900_MMI_0: Enum
    R5900_MMI_1: Enum
    R5900_MMI_2: Enum
    R5900_MMI_3: Enum
    R5900_MMI_PMFHL: Enum
    R5900_MMI_PMTHL: Enum
    ALL_MAX: Enum
