#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegCop1O32:
    fv0: Enum
    fv0f: Enum
    fv1: Enum
    fv1f: Enum
    ft0: Enum
    ft0f: Enum
    ft1: Enum
    ft1f: Enum
    ft2: Enum
    ft2f: Enum
    ft3: Enum
    ft3f: Enum
    fa0: Enum
    fa0f: Enum
    fa1: Enum
    fa1f: Enum
    ft4: Enum
    ft4f: Enum
    ft5: Enum
    ft5f: Enum
    fs0: Enum
    fs0f: Enum
    fs1: Enum
    fs1f: Enum
    fs2: Enum
    fs2f: Enum
    fs3: Enum
    fs3f: Enum
    fs4: Enum
    fs4f: Enum
    fs5: Enum
    fs5f: Enum
