#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022-2024 Decompollaborate
# SPDX-License-Identifier: MIT

# Automatically generated. DO NOT MODIFY

from __future__ import annotations
from .Enum import Enum
class RegCop1N32:
    fv0: Enum
    ft14: Enum
    fv1: Enum
    ft15: Enum
    ft0: Enum
    ft1: Enum
    ft2: Enum
    ft3: Enum
    ft4: Enum
    ft5: Enum
    ft6: Enum
    ft7: Enum
    fa0: Enum
    fa1: Enum
    fa2: Enum
    fa3: Enum
    fa4: Enum
    fa5: Enum
    fa6: Enum
    fa7: Enum
    fs0: Enum
    ft8: Enum
    fs1: Enum
    ft9: Enum
    fs2: Enum
    ft10: Enum
    fs3: Enum
    ft11: Enum
    fs4: Enum
    ft12: Enum
    fs5: Enum
    ft13: Enum
